/*  airguitar
 *
 * (C) 2012 by morecat_lab
 */

#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbdrv.h"
#include "oddebug.h"
#include "midi_code.h"
#include "ag.h"

/* EEPROMѿ */
static uchar __attribute__((section(".eeprom"))) e_conf = 1;
static uchar __attribute__((section(".eeprom"))) e_threshold = 80;

/*
  PIN ASSIGN

  PB3 : C3
  PB2 : D3
  PB1 : E3
  PB0 : F3
  PB4 : LED
  PB5 : (USB D-)
  PB6 : (USB D+)
  PB7 : (RESET)

  PA0 : C4
  PA1 : B3
  PA2 : A3
  PA3 : G3
  PA4 : SWITCH
  PA5 : ADC
  PA6 : 7TH
  PA7 : MINOR
 */

#define PORTB_DDR (0b00010000)
#define PORTA_DDR (0b00000000)

#define getPortBit(port, bit) ((*(port) & _BV(bit)) >> (bit))

/* debugging LED */
#define LED_OFF  { PORTB &= ~0x10; }
#define LED_ON   { PORTB |=  0x10; }
#define FLIP_LED { PORTB ^=  0x10; }

#define HW_CDC_BULK_OUT_SIZE     8
#define HW_CDC_BULK_IN_SIZE      8

enum {
    SEND_ENCAPSULATED_COMMAND = 0,
    GET_ENCAPSULATED_RESPONSE,
    SET_COMM_FEATURE,
    GET_COMM_FEATURE,
    CLEAR_COMM_FEATURE,
    SET_LINE_CODING = 0x20,
    GET_LINE_CODING,
    SET_CONTROL_LINE_STATE,
    SEND_BREAK
};

// This deviceDescrMIDI[] is based on 
// http://www.usb.org/developers/devclass_docs/midi10.pdf
// Appendix B. Example: Simple MIDI Adapter (Informative)

// B.1 Device Descriptor
static PROGMEM char deviceDescrMIDI[] = {	/* USB device descriptor */
  18,				/* sizeof(usbDescriptorDevice): length of descriptor in bytes */
  USBDESCR_DEVICE,		/* descriptor type */
  0x10, 0x01,			/* USB version supported */
  USB_CFG_DEVICE_CLASS,		/* device class: defined at interface level */
  USB_CFG_DEVICE_SUBCLASS,	/* subclass */
  0,				/* protocol */
  8,				/* max packet size */
  USB_CFG_VENDOR_ID,		/* 2 bytes */
  USB_CFG_DEVICE_ID,		/* 2 bytes */
  USB_CFG_DEVICE_VERSION,	/* 2 bytes */
  1,				/* manufacturer string index */
  2,				/* product string index */
  0,				/* serial number string index */
  1,				/* number of configurations */
};

// B.2 Configuration Descriptor
static PROGMEM char configDescrMIDI[] = { /* USB configuration descriptor */
  9,	   /* sizeof(usbDescrConfig): length of descriptor in bytes */
  USBDESCR_CONFIG,		/* descriptor type */
  101, 0,			/* total length of data returned (including inlined descriptors) */
  2,		      		/* number of interfaces in this configuration */
  1,				/* index of this configuration */
  0,				/* configuration name string index */
#if USB_CFG_IS_SELF_POWERED
    (1 << 7) | USBATTR_SELFPOWER, /* attributes */
#else
    (1 << 7),                   /* attributes */
#endif
  USB_CFG_MAX_BUS_POWER / 2,	/* max USB current in 2mA units */

// B.3 AudioControl Interface Descriptors
// The AudioControl interface describes the device structure (audio function topology) 
// and is used to manipulate the Audio Controls. This device has no audio function 
// incorporated. However, the AudioControl interface is mandatory and therefore both 
// the standard AC interface descriptor and the classspecific AC interface descriptor 
// must be present. The class-specific AC interface descriptor only contains the header 
// descriptor.

// B.3.1 Standard AC Interface Descriptor
// The AudioControl interface has no dedicated endpoints associated with it. It uses the 
// default pipe (endpoint 0) for all communication purposes. Class-specific AudioControl 
// Requests are sent using the default pipe. There is no Status Interrupt endpoint provided.
  /* descriptor follows inline: */
  9,			/* sizeof(usbDescrInterface): length of descriptor in bytes */
  USBDESCR_INTERFACE,	/* descriptor type */
  0,			/* index of this interface */
  0,			/* alternate setting for this interface */
  0,			/* endpoints excl 0: number of endpoint descriptors to follow */
  1,			/* */
  1,			/* */
  0,			/* */
  0,			/* string index for interface */

// B.3.2 Class-specific AC Interface Descriptor
// The Class-specific AC interface descriptor is always headed by a Header descriptor 
// that contains general information about the AudioControl interface. It contains all 
// the pointers needed to describe the Audio Interface Collection, associated with the 
// described audio function. Only the Header descriptor is present in this device 
// because it does not contain any audio functionality as such.
  /* descriptor follows inline: */
  9,			/* sizeof(usbDescrCDC_HeaderFn): length of descriptor in bytes */
  36,			/* descriptor type */
  1,			/* header functional descriptor */
  0x0, 0x01,		/* bcdADC */
  9, 0,			/* wTotalLength */
  1,			/* */
  1,			/* */

// B.4 MIDIStreaming Interface Descriptors

// B.4.1 Standard MS Interface Descriptor
  /* descriptor follows inline: */
  9,			/* length of descriptor in bytes */
  USBDESCR_INTERFACE,	/* descriptor type */
  1,			/* index of this interface */
  0,			/* alternate setting for this interface */
  2,			/* endpoints excl 0: number of endpoint descriptors to follow */
  1,			/* AUDIO */
  3,			/* MS */
  0,			/* unused */
  0,			/* string index for interface */

// B.4.2 Class-specific MS Interface Descriptor
  /* descriptor follows inline: */
  7,			/* length of descriptor in bytes */
  36,			/* descriptor type */
  1,			/* header functional descriptor */
  0x0, 0x01,		/* bcdADC */
  65, 0,			/* wTotalLength */

// B.4.3 MIDI IN Jack Descriptor
  /* descriptor follows inline: */
  6,			/* bLength */
  36,			/* descriptor type */
  2,			/* MIDI_IN_JACK desc subtype */
  1,			/* EMBEDDED bJackType */
  1,			/* bJackID */
  0,			/* iJack */

  /* descriptor follows inline: */
  6,			/* bLength */
  36,			/* descriptor type */
  2,			/* MIDI_IN_JACK desc subtype */
  2,			/* EXTERNAL bJackType */
  2,			/* bJackID */
  0,			/* iJack */

//B.4.4 MIDI OUT Jack Descriptor
  /* descriptor follows inline: */
  9,			/* length of descriptor in bytes */
  36,			/* descriptor type */
  3,			/* MIDI_OUT_JACK descriptor */
  1,			/* EMBEDDED bJackType */
  3,			/* bJackID */
  1,			/* No of input pins */
  2,			/* BaSourceID */
  1,			/* BaSourcePin */
  0,			/* iJack */

  /* descriptor follows inline: */
  9,			/* bLength of descriptor in bytes */
  36,			/* bDescriptorType */
  3,			/* MIDI_OUT_JACK bDescriptorSubtype */
  2,			/* EXTERNAL bJackType */
  4,			/* bJackID */
  1,			/* bNrInputPins */
  1,			/* baSourceID (0) */
  1,			/* baSourcePin (0) */
  0,			/* iJack */


// B.5 Bulk OUT Endpoint Descriptors

//B.5.1 Standard Bulk OUT Endpoint Descriptor
  /* descriptor follows inline: */
  9,			/* bLenght */
  USBDESCR_ENDPOINT,	/* bDescriptorType = endpoint */
  0x1,			/* bEndpointAddress OUT endpoint number 1 */
  3,			/* bmAttributes: 2:Bulk, 3:Interrupt endpoint */
  8, 0,			/* wMaxPacketSize */
  10,			/* bIntervall in ms */
  0,			/* bRefresh */
  0,			/* bSyncAddress */

// B.5.2 Class-specific MS Bulk OUT Endpoint Descriptor
  /* descriptor follows inline: */
  5,			/* bLength of descriptor in bytes */
  37,			/* bDescriptorType */
  1,			/* bDescriptorSubtype */
  1,			/* bNumEmbMIDIJack  */
  1,			/* baAssocJackID (0) */


//B.6 Bulk IN Endpoint Descriptors

//B.6.1 Standard Bulk IN Endpoint Descriptor
  /* descriptor follows inline: */
  9,			/* bLenght */
  USBDESCR_ENDPOINT,	/* bDescriptorType = endpoint */
  0x81,			/* bEndpointAddress IN endpoint number 1 */
  3,			/* bmAttributes: 2: Bulk, 3: Interrupt endpoint */
  8, 0,			/* wMaxPacketSize */
  10,			/* bIntervall in ms */
  0,			/* bRefresh */
  0,			/* bSyncAddress */

// B.6.2 Class-specific MS Bulk IN Endpoint Descriptor
  /* descriptor follows inline: */
  5,			/* bLength of descriptor in bytes */
  37,			/* bDescriptorType */
  1,			/* bDescriptorSubtype */
  1,			/* bNumEmbMIDIJack (0) */
  3,			/* baAssocJackID (0) */
};


uchar usbFunctionDescriptor(usbRequest_t * rq) {
  if (rq->wValue.bytes[1] == USBDESCR_DEVICE) {
    usbMsgPtr = (uchar *) deviceDescrMIDI;
    return sizeof(deviceDescrMIDI);
  } else {		/* must be config descriptor */
    usbMsgPtr = (uchar *) configDescrMIDI;
    return sizeof(configDescrMIDI);
  }
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

static uchar    sendEmptyFrame;
// static uchar    modeBuffer[7];

uchar usbFunctionSetup(uchar data[8]) {
  usbRequest_t    *rq = (void *)data;

  if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
    
    /*  Prepare bulk-in endpoint to respond to early termination   */
    if ((rq->bmRequestType & USBRQ_DIR_MASK) ==
	USBRQ_DIR_HOST_TO_DEVICE)
      sendEmptyFrame = 1;
  }
  return 0xff;
}

/*---------------------------------------------------------------------------*/
/* usbFunctionRead                                                           */
/*---------------------------------------------------------------------------*/

uchar usbFunctionRead( uchar *data, uchar len ) {
	data[0] = 0;
	data[1] = 0;
	data[2] = 0;
	data[3] = 0;
	data[4] = 0;
	data[5] = 0;
	data[6] = 0;
    return 7;
}

/*---------------------------------------------------------------------------*/
/* usbFunctionWrite                                                          */
/*---------------------------------------------------------------------------*/

uchar usbFunctionWrite(uchar * data, uchar len) {
  return 1;
}
/*---------------------------------------------------------------------------*/
/* usbFunctionWriteOut                                                       */
/*                                                                           */
/* this Function is called if a MIDI Out message (from PC) arrives.          */
/*                                                                           */
/*---------------------------------------------------------------------------*/

void usbFunctionWriteOut(uchar *data, uchar len ) {
  parseMidiMessage(data, len);
  return;			/* do nothing */
}

/*----------------------------------------------- */
/* åΥѡ*/
/*----------------------------------------------- */
void parseMidiMessage(uchar *data, uchar len) {
  uchar cin = (*data) & 0x0f;	/* CABLE NO̵뤹 */
  uchar note = (*(data+2)) & 0x7f;
  uchar vol  = (*(data+3)) & 0x7f;

  if (((*(data+1)) & 0x0f) != 0)			/* only channel 1 (=0) */
    return;

  switch(cin) {

    /* NOTE OFF */
  case 8:
    // NoteOFF(note);
    LED_OFF;
    break;

    /* NOTE ON */
  case 9:
    if( vol == 0){
      // NoteOFF(note);
      LED_OFF;
    } else {
      // NoteON(note, vol);
      LED_ON;
    }
    break;

    /* CONTROL CHANGE */
  case 11:			
    if (note == 122) {		/* local control */
      /* ͤѹ */
      vol = *(data+3);
      eeprom_busy_wait();	/* EEPROMǥˤʤޤԤ */
      eeprom_write_byte(&e_threshold, vol); /* EEPROM˽񤭹 */
    } else if ((note == 120) || (note == 123)) {
      /* all note off */
      initializeKeyStatus();
    } else if (note == 121) {
      /* 122 reset all controller */
      initializeKeyStatus();
    }
    break;

    //  case 0:			/* MISC FUNCTION CODE */
    //  case 1:			/* CABLE EVENT */
    //  case 2:			/* TWO-BYTES SYSTEM COMMON */
    //  case 3:			/* THREE-BYTES SYSTEM COMMON */
    //  case 4:			/* SYS EX START */
    //  case 5:			/* SINGLE BYTE SYSTEM COMMON */
    //  case 6:			/* SYSTEM EX end (with 2 byte) */
    //  case 7:			/* SYSTEM EX end (with 3 byte) */
    //  case 10:		/* POLY KEY PRESS */
    //  case 12:		/* PROGRAM CHANGE */
    //  case 13:		/* CHANNEL PRESSURE */
    //  case 14:		/* PITCH BEND */
    //  case 15:			/* SINGLE BYTE */
    //    break;
  }

  if (len > 4) {
    parseMidiMessage(data+4, len-4);
  }
}

/*----------------------------------------------- */
/* Key Event */

/* keyEventQueue */
struct keyEvent {
  uchar key;
  //  uchar onOff;
};

#define KEY_OFF 0
#define KEY_ON 1
#define MAX_KEY_EVENT 32
#define KEY_EVENT_QUEUE_MASK (MAX_KEY_EVENT-1)
#define KEY_COUNT_TH 10
#define TRIG_TH 3

struct keyEvent keyEventQueue[MAX_KEY_EVENT];
uchar urptr, uwptr, iwptr;

uchar conf;
uchar thold;

static inline uchar keyEventsFree (struct keyEvent **ptr) {
  return (MAX_KEY_EVENT - uwptr - 1);
}

void keyEventAppend(uchar key, uchar onOff) {
  uchar iwnxt;
  iwnxt = (iwptr +1 ) & KEY_EVENT_QUEUE_MASK;
  if (iwnxt == urptr )
    return;			/* buffer overflow */
  keyEventQueue[iwptr].key = key | (onOff << 7);
  iwptr = iwnxt;
}

static inline uchar keyEventAvailable(struct keyEvent **ptr) {
  *ptr = &keyEventQueue[urptr];
  if(iwptr >= urptr) {
    return iwptr - urptr;
  } else {  /* buffer end is between read and write pointer, return continuous range */
    return MAX_KEY_EVENT - urptr;
  }
}

/* The following function must be called after uartRxBytesAvailable() to
 * remove the bytes from the receiver buffer.
 */
static inline void  keyEventDidReadEvents(uchar numEvents) {
  urptr = (urptr + numEvents) & KEY_EVENT_QUEUE_MASK;
}

static inline void keyEventQueueClear() {
  urptr = uwptr = 0;
}

/* PORT MAP TABLE */
struct portInfo_t {
  uchar note_no;
  volatile uchar *port;
  uchar bit;
  uchar status;
  uchar shiftStatus;
  uchar keyCountButton;
} portInfoIn[] = {
  {MIDI_NOTE_C4, &PINB, 3, 0, 0, 0},		/* IX:00 C3 */
  {MIDI_NOTE_D4, &PINB, 2, 0, 0, 0},		/* IX:02 D3 */
  {MIDI_NOTE_E4, &PINB, 1, 0, 0, 0},		/* IX:04 E3 */
  {MIDI_NOTE_F4, &PINB, 0, 0, 0, 0},		/* IX:05 F3 */
  {MIDI_NOTE_G4, &PINA, 3, 0, 0, 0},		/* IX:07 G3 */
  {MIDI_NOTE_A4, &PINA, 2, 0, 0, 0},		/* IX:09 A3 */
  {MIDI_NOTE_B4, &PINA, 1, 0, 0, 0},		/* IX:11 B3 */
  {MIDI_NOTE_C5, &PINA, 0, 0, 0, 0},		/* IX:11 B3 */
};

/* TRIGGER STATUS */
uchar trig_status = 0;

#define MAX_KEYS 8

void portInit() {
  DDRB  |= PORTB_DDR;
  PORTB |= 0b00001111;			/* All pull-up, LED OFF */
  DDRA  =  PORTA_DDR;			/* ALL INPUT */
  PORTA =  0b11011111;			/* ALL pull-up, but PA5 */
}

void initializeKeyStatus() {
  uchar i;
  for (i = 0 ; i < MAX_KEYS ; i++) {
    portInfoIn[i].keyCountButton = 0;
    portInfoIn[i].status = 1;
    portInfoIn[i].shiftStatus = 0;
  }
}

/* PA4 == H (COHRD MODE) L (NORMAL MODE) */
#define CHORD_MODE_CHECK() (((PINA) & 0x10) == 0)

/* PA7 == H (NORMAL) L (MINOR CHORD) */
#define CHORD_SHIFT_MINOR() (((PINA) & 0x80) == 0)

/* PA6 == H (NORMAL) L (7th CHORD) */
#define CHORD_SHIFT_7TH() (((PINA) & 0x40) == 0)

void sendKeyEvent(uchar keyIndex, uchar noteOnOff) {
  uchar note = portInfoIn[keyIndex].note_no;
  uchar shiftStatus;
  if (noteOnOff == 1) {		/* only check shiftkeys at key-on */
    portInfoIn[keyIndex].shiftStatus = 0;
    if (CHORD_MODE_CHECK()) {
      portInfoIn[keyIndex].shiftStatus |= 1;
    }
    if (CHORD_SHIFT_MINOR()) {
      portInfoIn[keyIndex].shiftStatus |= 2;
    }
    if (CHORD_SHIFT_7TH()) {
      portInfoIn[keyIndex].shiftStatus |= 4;
    }
    /* for key-off old shiftkey information is used */
  }

  shiftStatus = portInfoIn[keyIndex].shiftStatus;
  if (bit_is_set(shiftStatus, 0)) { /* NON CHORD MODE */
    keyEventAppend(note, noteOnOff);
  } else {
    if (bit_is_set(shiftStatus, 1)) { /* CHORD_MINOR */
      if (bit_is_set(shiftStatus, 2)) { /* CHORD MINOR + 7TH */
	keyEventAppend(note, noteOnOff);
	keyEventAppend(note + 3, noteOnOff);
	keyEventAppend(note + 7, noteOnOff);
	keyEventAppend(note + 10, noteOnOff);
      } else {			/* CHORD MINOR */
	keyEventAppend(note, noteOnOff);
	keyEventAppend(note + 3, noteOnOff);
	keyEventAppend(note + 7, noteOnOff);
      }
    } else if (bit_is_set(shiftStatus, 2)) { /* CHORD 7TH */
      keyEventAppend(note, noteOnOff);
      keyEventAppend(note + 4, noteOnOff);
      keyEventAppend(note + 7, noteOnOff);
      keyEventAppend(note + 10, noteOnOff);
    } else {			/* NORMAL CHORD */
      keyEventAppend(note, noteOnOff);
      keyEventAppend(note + 4, noteOnOff);
      keyEventAppend(note + 7, noteOnOff);
    }
  }
}

/* ݥΥ */
static inline void scanKeyPressed(void) {
  uchar x;
  uchar i;

  for (i = 0; i < MAX_KEYS ; i++) {
    if (portInfoIn[i].keyCountButton > 0) {
      /* in the wait loop for 2nd check */
      portInfoIn[i].keyCountButton --;
      if (portInfoIn[i].keyCountButton == 0) {
	/* now, we check 2nd time */
	x = getPortBit(portInfoIn[i].port, portInfoIn[i].bit);	/* read PORT */
	if (x != portInfoIn[i].status) { /* status changed */
	  if (trig_status == 1) {
	    if (x == 0) {	/* key on */
	      sendKeyEvent(i, KEY_ON);
	      LED_ON;
	    } else {		/* key off */
	      sendKeyEvent(i, KEY_OFF);
	      LED_OFF;
	    }
	  }
	  portInfoIn[i].status = x;
	}
      }
    } else {
      /* 1st key check */
      x = getPortBit(portInfoIn[i].port, portInfoIn[i].bit);	/* read PORT */
      if (x != portInfoIn[i].status) {
	portInfoIn[i].keyCountButton = KEY_COUNT_TH;
      }
    }
  } /* for */
}

/*
  AD conversion
 */

/* Ѵ */
#define ADCStart() { ADCSRA |= 0x10;  ADCSRA |= 0x40; }

void setupAdc() {
  ADCSRA = 0x87;		/* AD enable, 1/128 clock  */
  ADMUX  = 0x04;		/* use PA5 */
  ADCStart();
}

void newTriggerEvent(uint8_t onOff) {
  uint8_t i;
  for (i = 0 ; i < MAX_KEYS ; i++ ) {
    if (onOff == 1) {		/* trigger on */
      if (portInfoIn[i].status == 0) { /* KEY == ON */
	sendKeyEvent(i, KEY_ON);
	LED_ON;
      }
    } else {			/* trigger off */
      if (portInfoIn[i].status == 0) { /* KEY == ON */
	sendKeyEvent(i, KEY_OFF);
	LED_OFF;
      }
    }
  } /* for */
}

void checkAdc() {
  static uint8_t count = 0;
  static uint8_t preVal = 0;
  uint16_t adval;
  uint8_t val;
  if ((ADCSRA & 0x10) != 0) {
    adval = (ADC >> 2);
    ADCStart();
    val = (adval > thold) ? 1 : 0;
    if (count == 0) {		/* 1st trigger */
      if (val != preVal) {
	count = TRIG_TH;
      }
    } else {			/* 2nd or more */
      count --;
      if (count == 0) {
	if (val != preVal) {	/* state changes */
	  if (val == 1) {
	    trig_status = 1;
	    // LED_ON;
	  } else {
	    trig_status = 0;
	    // LED_OFF;
	  }
	  preVal = val;
	  newTriggerEvent(trig_status);
	}
      }
    }
  }
}


/* ------------------------------------------------------------------------- */
/* ------------------------ Oscillator Calibration ------------------------- */
/* ------------------------------------------------------------------------- */

#define abs(x) ((x) > 0 ? (x) : (-x))

// Called by V-USB after device reset
void hadUsbReset() {
    int frameLength, targetLength = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);
    int bestDeviation = 9999;
    uchar trialCal, bestCal = 0, step, region;

    // do a binary search in regions 0-127 and 128-255 to get optimum OSCCAL
    for(region = 0; region <= 1; region++) {
        frameLength = 0;
        trialCal = (region == 0) ? 0 : 128;
        
        for(step = 64; step > 0; step >>= 1) { 
            if(frameLength < targetLength) // true for initial iteration
                trialCal += step; // frequency too low
            else
                trialCal -= step; // frequency too high
                
            OSCCAL = trialCal;
            frameLength = usbMeasureFrameLength();
            
            if(abs(frameLength-targetLength) < bestDeviation) {
                bestCal = trialCal; // new optimum found
                bestDeviation = abs(frameLength -targetLength);
            }
        }
    }

    OSCCAL = bestCal;
}

void calibrate() {
  uchar i;
  usbDeviceDisconnect(); // enforce re-enumeration
  for(i = 0; i<250; i++) { // wait 500 ms
    wdt_reset(); // keep the watchdog happy
    _delay_ms(2);
  }
  usbDeviceConnect();	
}

int main(void) {
  uchar midiMsg[8];
  uchar i, iii;

  /* conf, tholdEEPROMɤ߹ꤹ */
  eeprom_busy_wait();
  conf = eeprom_read_byte(&e_conf);
  eeprom_busy_wait();
  thold = eeprom_read_byte(&e_threshold);

  /* ݡȤ*/
  portInit();

  /* WDTư */
  wdt_enable(WDTO_1S);

  usbInit();
  calibrate();

  sendEmptyFrame = 0;

  setupAdc();
  initializeKeyStatus();

  sei();

  for(;;){
    wdt_reset();
    usbPoll();

    checkAdc();
    scanKeyPressed();

    /* USB */
    if (usbInterruptIsReady()) {
      uchar eventsRead;
      struct keyEvent *data;
      
      eventsRead = keyEventAvailable(&data);
      if(eventsRead > 0 || sendEmptyFrame){
	if(eventsRead >= 2) {	/* up to 2 events can be sent */
	  eventsRead = 2;
	  /* send an empty block after last data block to indicate transfer end */
	  sendEmptyFrame  = 1;
	} else {
	  sendEmptyFrame  = 0;
	}
	iii = 0;
	for (i = 0; i < eventsRead ; i++) {
	  if (((data ->key) & 0x80) == (KEY_ON << 7)) { /* NOTE ON */
	    midiMsg[iii++] = 0x09;
	    midiMsg[iii++] = 0x90;
	    midiMsg[iii++] = (data->key) & 0x7f;
	    midiMsg[iii++] = 0x7f;
	  } else {		/* NOTE OFF */
	    midiMsg[iii++] = 0x08;
	    midiMsg[iii++] = 0x80;
	    midiMsg[iii++] = (data->key) & 0x7f;
	    midiMsg[iii++] = 0x00;
	  }
	  keyEventDidReadEvents(1);
	  keyEventAvailable(&data); /* get next event data */
	}
	usbSetInterrupt(midiMsg, eventsRead * 4);
      }
    } /* usbInterruptIsReady() */

  } /* main event loop */
}

/* EOF */
